@@ -8,5 +8,4 @@ class GISConfig(AppConfig):
     verbose_name = _("GIS")
 
     def ready(self):
-        if 'geojson' not in serializers.BUILTIN_SERIALIZERS:
-            serializers.BUILTIN_SERIALIZERS['geojson'] = "django.contrib.gis.serializers.geojson"
+        serializers.BUILTIN_SERIALIZERS.setdefault('geojson', 'django.contrib.gis.serializers.geojson')
